@@ -1,5 +1,5 @@
 // SPDX-License-Identifier: BSD-2-Clause
-/* Copyright (C) 2016 - 2020 Intel Corporation. */
+/* Copyright (C) 2016 - 2021 Intel Corporation. */
 
 #pragma once
 #ifdef __cplusplus
@@ -44,6 +44,7 @@ extern "C" {
 #define jemk_malloc_usable_size     JE_SYMBOL(malloc_usable_size)
 #define jemk_arenalookupx           JE_SYMBOL(arenalookupx)
 #define jemk_check_reallocatex      JE_SYMBOL(check_reallocatex)
+#define jemk_malloc_stats_print     JE_SYMBOL(malloc_stats_print)
 
 enum memkind_const_private {
     MEMKIND_NAME_LENGTH_PRIV = 64
@@ -96,10 +97,20 @@ struct memkind_config {
     memkind_mem_usage_policy policy; //kind memory usage policy
 };
 
+typedef enum memkind_node_variant_t {
+    NODE_VARIANT_MULTIPLE = 0,
+    NODE_VARIANT_SINGLE = 1,
+    NODE_VARIANT_MAX = 2,
+    NODE_VARIANT_ALL = NODE_VARIANT_MAX,
+    NODE_VARIANT_MAX_EXT = 3
+} memkind_node_variant_t;
+
 void memkind_init(memkind_t kind, bool check_numa);
 
 void *kind_mmap(struct memkind *kind, void *addr, size_t size);
 
+char *memkind_get_env(const char *name);
+
 #ifdef __cplusplus
 }
 #endif
