@@ -1,34 +1,16 @@
-/*
- * Copyright (C) 2014 - 2019 Intel Corporation.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 1. Redistributions of source code must retain the above copyright notice(s),
- *    this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright notice(s),
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
- * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-2-Clause
+/* Copyright (C) 2014 - 2021 Intel Corporation. */
+
 #define MEMKIND_VERSION_MAJOR 1
-#define MEMKIND_VERSION_MINOR 10
+#define MEMKIND_VERSION_MINOR 11
 #define MEMKIND_VERSION_PATCH 0
 
 #include <memkind.h>
 #include <memkind/internal/memkind_default.h>
 #include <memkind/internal/memkind_hugetlb.h>
 #include <memkind/internal/memkind_arena.h>
+#include <memkind/internal/memkind_capacity.h>
+#include <memkind/internal/memkind_local.h>
 #include <memkind/internal/memkind_hbw.h>
 #include <memkind/internal/memkind_regular.h>
 #include <memkind/internal/memkind_gbtlb.h>
@@ -49,6 +31,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <limits.h>
 #include <assert.h>
 #include <pthread.h>
 #include <errno.h>
@@ -56,6 +39,30 @@
 #include <fcntl.h>
 #include <unistd.h>
 
+#ifdef MEMKIND_ENABLE_HEAP_MANAGER
+#define m_detect_kind(ptr)                      heap_manager_detect_kind(ptr)
+#define m_free(ptr)                             heap_manager_free(ptr)
+#define m_realloc(ptr, size)                    heap_manager_realloc(ptr, size)
+#define m_usable_size(ptr)                      heap_manager_malloc_usable_size(ptr)
+#define m_defrag_reallocate(ptr)                heap_manager_defrag_reallocate(ptr)
+#define m_get_global_stat(stat, value)          heap_manager_get_stat(stat, value)
+#define m_update_cached_stats                   heap_manager_update_cached_stats
+#define m_init                                  heap_manager_init
+#define m_set_bg_threads(state)                 heap_manager_set_bg_threads(state)
+#define m_stats_print(write_cb, cbopaque, opts) heap_manager_stats_print(write_cb, cbopaque, opts)
+#else
+#define m_detect_kind(ptr)                      memkind_arena_detect_kind(ptr)
+#define m_free(ptr)                             memkind_arena_free_with_kind_detect(ptr)
+#define m_realloc(ptr, size)                    memkind_arena_realloc_with_kind_detect(ptr, size)
+#define m_usable_size(ptr)                      memkind_default_malloc_usable_size(NULL, ptr)
+#define m_defrag_reallocate(ptr)                memkind_arena_defrag_reallocate_with_kind_detect(ptr)
+#define m_get_global_stat(stat, value)          memkind_arena_get_global_stat(stat, value)
+#define m_update_cached_stats                   memkind_arena_update_cached_stats
+#define m_init                                  memkind_arena_init
+#define m_set_bg_threads(state)                 memkind_arena_set_bg_threads(state)
+#define m_stats_print(write_cb, cbopaque, opts) memkind_arena_stats_print(write_cb, cbopaque, opts)
+#endif
+
 /* Clear bits in x, but only this specified in mask. */
 #define CLEAR_BIT(x, mask) ((x) &= (~(mask)))
 
@@ -184,6 +191,69 @@ static struct memkind MEMKIND_DAX_KMEM_PREFERRED_STATIC = {
     .init_once = PTHREAD_ONCE_INIT,
 };
 
+static struct memkind MEMKIND_DAX_KMEM_INTERLEAVE_STATIC = {
+    .ops = &MEMKIND_DAX_KMEM_INTERLEAVE_OPS,
+    .partition = MEMKIND_PARTITION_DAX_KMEM_INTERLEAVE,
+    .name = "memkind_dax_kmem_interleave",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_HIGHEST_CAPACITY_STATIC = {
+    .ops = &MEMKIND_HIGHEST_CAPACITY_OPS,
+    .partition = MEMKIND_PARTITION_HIGHEST_CAPACITY,
+    .name = "memkind_highest_capacity",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_HIGHEST_CAPACITY_PREFERRED_STATIC = {
+    .ops = &MEMKIND_HIGHEST_CAPACITY_PREFERRED_OPS,
+    .partition = MEMKIND_PARTITION_HIGHEST_CAPACITY_PREFERRED,
+    .name = "memkind_highest_capacity_preferred",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC = {
+    .ops = &MEMKIND_HIGHEST_CAPACITY_LOCAL_OPS,
+    .partition = MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL,
+    .name = "memkind_highest_capacity_local",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_STATIC = {
+    .ops = &MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_OPS,
+    .partition = MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL_PREFERRED,
+    .name = "memkind_highest_capacity_local_preferred",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_LOWEST_LATENCY_LOCAL_STATIC = {
+    .ops = &MEMKIND_LOWEST_LATENCY_LOCAL_OPS,
+    .partition = MEMKIND_PARTITION_LOWEST_LATENCY_LOCAL,
+    .name = "memkind_lowest_latency_local",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_STATIC = {
+    .ops = &MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_OPS,
+    .partition = MEMKIND_PARTITION_LOWEST_LATENCY_LOCAL_PREFERRED,
+    .name = "memkind_lowest_latency_local_preferred",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_HIGHEST_BANDWIDTH_LOCAL_STATIC = {
+    .ops = &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_OPS,
+    .partition = MEMKIND_PARTITION_HIGHEST_BANDWIDTH_LOCAL,
+    .name = "memkind_highest_bandwidth_local",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
+static struct memkind MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_STATIC = {
+    .ops = &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_OPS,
+    .partition = MEMKIND_PARTITION_HIGHEST_BANDWIDTH_LOCAL_PREFERRED,
+    .name = "memkind_highest_bandwidth_local_preferred",
+    .init_once = PTHREAD_ONCE_INIT,
+};
+
 MEMKIND_EXPORT struct memkind *MEMKIND_DEFAULT = &MEMKIND_DEFAULT_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_HUGETLB = &MEMKIND_HUGETLB_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_INTERLEAVE = &MEMKIND_INTERLEAVE_STATIC;
@@ -209,6 +279,24 @@ MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_ALL =
         &MEMKIND_DAX_KMEM_ALL_STATIC;
 MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_PREFERRED =
         &MEMKIND_DAX_KMEM_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_DAX_KMEM_INTERLEAVE =
+        &MEMKIND_DAX_KMEM_INTERLEAVE_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY =
+        &MEMKIND_HIGHEST_CAPACITY_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_PREFERRED =
+        &MEMKIND_HIGHEST_CAPACITY_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_LOCAL =
+        &MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED =
+        &MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_LOWEST_LATENCY_LOCAL =
+        &MEMKIND_LOWEST_LATENCY_LOCAL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED =
+        &MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_BANDWIDTH_LOCAL =
+        &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_STATIC;
+MEMKIND_EXPORT struct memkind *MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED =
+        &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_STATIC;
 
 struct memkind_registry {
     struct memkind *partition_map[MEMKIND_MAX_KIND];
@@ -235,6 +323,15 @@ static struct memkind_registry memkind_registry_g = {
         [MEMKIND_PARTITION_DAX_KMEM] = &MEMKIND_DAX_KMEM_STATIC,
         [MEMKIND_PARTITION_DAX_KMEM_ALL] = &MEMKIND_DAX_KMEM_ALL_STATIC,
         [MEMKIND_PARTITION_DAX_KMEM_PREFERRED] = &MEMKIND_DAX_KMEM_PREFERRED_STATIC,
+        [MEMKIND_PARTITION_DAX_KMEM_INTERLEAVE] = &MEMKIND_DAX_KMEM_INTERLEAVE_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_CAPACITY] = &MEMKIND_HIGHEST_CAPACITY_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_CAPACITY_PREFERRED] = &MEMKIND_HIGHEST_CAPACITY_PREFERRED_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL] = &MEMKIND_HIGHEST_CAPACITY_LOCAL_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_CAPACITY_LOCAL_PREFERRED] = &MEMKIND_HIGHEST_CAPACITY_LOCAL_PREFERRED_STATIC,
+        [MEMKIND_PARTITION_LOWEST_LATENCY_LOCAL] = &MEMKIND_LOWEST_LATENCY_LOCAL_STATIC,
+        [MEMKIND_PARTITION_LOWEST_LATENCY_LOCAL_PREFERRED] = &MEMKIND_LOWEST_LATENCY_LOCAL_PREFERRED_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_BANDWIDTH_LOCAL] = &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_STATIC,
+        [MEMKIND_PARTITION_HIGHEST_BANDWIDTH_LOCAL_PREFERRED] = &MEMKIND_HIGHEST_BANDWIDTH_LOCAL_PREFERRED_STATIC,
     },
     MEMKIND_NUM_BASE_KIND,
     PTHREAD_MUTEX_INITIALIZER
@@ -271,8 +368,8 @@ static int validate_flags_bits(memkind_bits_t flags)
 
 static int validate_policy(memkind_policy_t policy)
 {
-    if((policy >= 0) && (policy < MEMKIND_POLICY_MAX_VALUE)) return 0;
-    return -1;
+    if (policy >= MEMKIND_POLICY_MAX_VALUE) return -1;
+    return 0;
 }
 
 struct create_args {
@@ -350,7 +447,7 @@ static void memkind_destroy_dynamic_kind_from_register(unsigned int i,
     if (i >= MEMKIND_NUM_BASE_KIND) {
         memkind_registry_g.partition_map[i] = NULL;
         --memkind_registry_g.num_kind;
-        free(kind);
+        jemk_free(kind);
     }
 }
 
@@ -375,7 +472,7 @@ MEMKIND_EXPORT int memkind_destroy_kind(memkind_t kind)
 
 MEMKIND_EXPORT memkind_t memkind_detect_kind(void *ptr)
 {
-    return heap_manager_detect_kind(ptr);
+    return m_detect_kind(ptr);
 }
 
 /* Declare weak symbols for allocator decorators */
@@ -470,7 +567,7 @@ MEMKIND_EXPORT void memkind_error_message(int err, char *msg, size_t size)
 void memkind_init(memkind_t kind, bool check_numa)
 {
     log_info("Initializing kind %s.", kind->name);
-    heap_manager_init(kind);
+    m_init(kind);
     if (check_numa) {
         int err = numa_available();
         if (err) {
@@ -480,6 +577,15 @@ void memkind_init(memkind_t kind, bool check_numa)
     }
 }
 
+char *memkind_get_env(const char *name)
+{
+#ifdef MEMKIND_HAVE_SECURE_GETENV
+    return secure_getenv(name);
+#else
+    return getenv(name);
+#endif
+}
+
 static void nop(void) {}
 
 static int memkind_create(struct memkind_ops *ops, const char *name,
@@ -520,7 +626,7 @@ static int memkind_create(struct memkind_ops *ops, const char *name,
             goto exit;
         }
     }
-    *kind = (struct memkind *)calloc(1, sizeof(struct memkind));
+    *kind = (struct memkind *)jemk_calloc(1, sizeof(struct memkind));
     if (!*kind) {
         err = MEMKIND_ERROR_MALLOC;
         log_err("calloc() failed.");
@@ -530,7 +636,7 @@ static int memkind_create(struct memkind_ops *ops, const char *name,
     (*kind)->partition = memkind_registry_g.num_kind;
     err = ops->create(*kind, ops, name);
     if (err) {
-        free(*kind);
+        jemk_free(*kind);
         goto exit;
     }
     memkind_registry_g.partition_map[id_kind] = *kind;
@@ -546,25 +652,34 @@ exit:
     return err;
 }
 
+static int memkind_use_other_heap_manager(void)
+{
+#ifdef MEMKIND_ENABLE_HEAP_MANAGER
+    const char *env = memkind_get_env("MEMKIND_HEAP_MANAGER");
+    if (env && strcmp(env, "TBB") == 0) {
+        load_tbb_symbols();
+        return 1;
+    }
+#endif
+    return 0;
+}
+
 #ifdef __GNUC__
 __attribute__((constructor))
 #endif
 static void memkind_construct(void)
 {
-    const char *env = secure_getenv("MEMKIND_HEAP_MANAGER");
-    if (env && strcmp(env, "TBB") == 0) {
-        load_tbb_symbols();
-    } else {
-        env = secure_getenv("MEMKIND_BACKGROUND_THREAD_LIMIT");
+    if (!memkind_use_other_heap_manager()) {
+        const char *env = memkind_get_env("MEMKIND_BACKGROUND_THREAD_LIMIT");
         if (env) {
             char *end;
             errno = 0;
-            size_t thread_limit = strtoull(env, &end, 10);
-            if (*end != '\0' || errno != 0 ) {
-                log_err("Error on parsing value MEMKIND_BACKGROUND_THREAD_LIMIT");
-                return;
+            size_t thread_limit = strtoul(env, &end, 10);
+            if (thread_limit > UINT_MAX || *end != '\0' || errno != 0) {
+                log_fatal("Error: Wrong value of MEMKIND_BACKGROUND_THREAD_LIMIT=%s", env);
+                abort();
             }
-            memkind_arena_enable_background_threads(thread_limit);
+            memkind_arena_set_max_bg_threads(thread_limit);
         }
     }
 }
@@ -614,7 +729,7 @@ MEMKIND_EXPORT size_t memkind_malloc_usable_size(struct memkind *kind,
                                                  void *ptr)
 {
     if (!kind) {
-        return heap_manager_malloc_usable_size(ptr);
+        return m_usable_size(ptr);
     } else {
         return kind->ops->malloc_usable_size(kind, ptr);
     }
@@ -704,7 +819,7 @@ MEMKIND_EXPORT void *memkind_realloc(struct memkind *kind, void *ptr,
 #endif
 
     if (!kind) {
-        result = heap_manager_realloc(ptr, size);
+        result = m_realloc(ptr, size);
     } else {
         pthread_once(&kind->init_once, kind->ops->init_once);
         result = kind->ops->realloc(kind, ptr, size);
@@ -727,7 +842,7 @@ MEMKIND_EXPORT void memkind_free(struct memkind *kind, void *ptr)
     }
 #endif
     if (!kind) {
-        heap_manager_free(ptr);
+        m_free(ptr);
     } else {
         pthread_once(&kind->init_once, kind->ops->init_once);
         kind->ops->free(kind, ptr);
@@ -740,48 +855,6 @@ MEMKIND_EXPORT void memkind_free(struct memkind *kind, void *ptr)
 #endif
 }
 
-static int memkind_tmpfile(const char *dir, int *fd)
-{
-    static char template[] = "/memkind.XXXXXX";
-    int err = MEMKIND_SUCCESS;
-    int oerrno;
-    int dir_len = strlen(dir);
-
-    if (dir_len > PATH_MAX) {
-        log_err("Could not create temporary file: too long path.");
-        return MEMKIND_ERROR_INVALID;
-    }
-
-    char fullname[dir_len + sizeof (template)];
-    (void) strcpy(fullname, dir);
-    (void) strcat(fullname, template);
-
-    sigset_t set, oldset;
-    sigfillset(&set);
-    (void) sigprocmask(SIG_BLOCK, &set, &oldset);
-
-    if ((*fd = mkstemp(fullname)) < 0) {
-        log_err("Could not create temporary file: errno=%d.", errno);
-        err = MEMKIND_ERROR_INVALID;
-        goto exit;
-    }
-
-    (void) unlink(fullname);
-    (void) sigprocmask(SIG_SETMASK, &oldset, NULL);
-
-    return err;
-
-exit:
-    oerrno = errno;
-    (void) sigprocmask(SIG_SETMASK, &oldset, NULL);
-    if (*fd != -1) {
-        (void) close(*fd);
-    }
-    *fd = -1;
-    errno = oerrno;
-    return err;
-}
-
 MEMKIND_EXPORT struct memkind_config *memkind_config_new(void)
 {
     struct memkind_config *cfg = (struct memkind_config *)malloc(sizeof(
@@ -830,7 +903,7 @@ MEMKIND_EXPORT int memkind_create_pmem(const char *dir, size_t max_size,
     int fd = -1;
     char name[16];
 
-    int err = memkind_tmpfile(dir, &fd);
+    int err = memkind_pmem_create_tmpfile(dir, &fd);
     if (err) {
         goto exit;
     }
@@ -848,6 +921,11 @@ MEMKIND_EXPORT int memkind_create_pmem(const char *dir, size_t max_size,
     priv->offset = 0;
     priv->current_size = 0;
     priv->max_size = max_size;
+    priv->dir = jemk_malloc(strlen(dir)+1);
+    if (!priv->dir) {
+        goto exit;
+    }
+    memcpy(priv->dir, dir, strlen(dir));
 
     return err;
 
@@ -896,13 +974,13 @@ MEMKIND_EXPORT int memkind_get_kind_by_partition(int partition,
 
 MEMKIND_EXPORT int memkind_update_cached_stats(void)
 {
-    return heap_manager_update_cached_stats();
+    return m_update_cached_stats();
 }
 
 MEMKIND_EXPORT void *memkind_defrag_reallocate(memkind_t kind, void *ptr)
 {
     if (!kind) {
-        return heap_manager_defrag_reallocate(ptr);
+        return m_defrag_reallocate(ptr);
     } else {
         return kind->ops->defrag_reallocate(kind, ptr);
     }
@@ -917,8 +995,24 @@ MEMKIND_EXPORT int memkind_get_stat(memkind_t kind, memkind_stat_type stat,
     }
 
     if (!kind) {
-        return heap_manager_get_stat(stat, value);
+        return m_get_global_stat(stat, value);
     } else {
         return kind->ops->get_stat(kind, stat, value);
     }
 }
+
+MEMKIND_EXPORT int memkind_check_dax_path(const char *pmem_dir)
+{
+    return memkind_pmem_validate_dir(pmem_dir);
+}
+
+MEMKIND_EXPORT int memkind_set_bg_threads(bool state)
+{
+    return m_set_bg_threads(state);
+}
+
+MEMKIND_EXPORT int memkind_stats_print(void (*write_cb) (void *, const char *),
+                                       void *cbopaque, memkind_stat_print_opt opts)
+{
+    return m_stats_print(write_cb, cbopaque, opts);
+}
