@@ -1,26 +1,5 @@
-/*
- * Copyright (C) 2014 - 2018 Intel Corporation.
- * All rights reserved.
- *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions are met:
- * 1. Redistributions of source code must retain the above copyright notice(s),
- *    this list of conditions and the following disclaimer.
- * 2. Redistributions in binary form must reproduce the above copyright notice(s),
- *    this list of conditions and the following disclaimer in the documentation
- *    and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER(S) ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO
- * EVENT SHALL THE COPYRIGHT HOLDER(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
- * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
- * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- */
+// SPDX-License-Identifier: BSD-2-Clause
+/* Copyright (C) 2014 - 2020 Intel Corporation. */
 
 #include <memkind/internal/memkind_hbw.h>
 #include "allocator_perf_tool/HugePageOrganizer.hpp"
@@ -30,6 +9,7 @@
 #include <vector>
 #include <numa.h>
 #include <numaif.h>
+#include <unistd.h>
 
 trial_t TrialGenerator :: create_trial_tuple(alloc_api_t api,
                                              size_t size,
@@ -93,7 +73,7 @@ void TrialGenerator :: generate_size_2bytes_2KB_2MB(alloc_api_t api)
             create_trial_tuple(
                 api,size[i],
                 32,
-                4096,
+                sysconf(_SC_PAGESIZE),
                 MEMKIND_HBW,
                 -1
             )
@@ -201,7 +181,7 @@ void TrialGenerator :: run(int num_bandwidth, std::vector<int> &bandwidth)
                 fprintf (stdout,"Allocating %zd bytes using hbw_memalign_psize\n",
                          trial_vec[i].size);
                 hbw_pagesize_t psize;
-                if (trial_vec[i].page_size == 4096)
+                if (trial_vec[i].page_size == (size_t)sysconf(_SC_PAGESIZE))
                     psize = HBW_PAGESIZE_4KB;
                 else if (trial_vec[i].page_size == 2097152)
                     psize = HBW_PAGESIZE_2MB;
@@ -283,7 +263,7 @@ void TGTest :: SetUp()
 {
     size_t node;
     char *hbw_nodes_env, *endptr;
-    tgen = std::move(std::unique_ptr<TrialGenerator>(new TrialGenerator()));
+    tgen = std::unique_ptr<TrialGenerator>(new TrialGenerator());
 
     hbw_nodes_env = getenv("MEMKIND_HBW_NODES");
     if (hbw_nodes_env) {
@@ -296,7 +276,7 @@ void TGTest :: SetUp()
         while (*endptr == ':') {
             hbw_nodes_env = endptr + 1;
             node = strtol(hbw_nodes_env, &endptr, 10);
-            if (endptr != hbw_nodes_env && node >= 0 && node < num_bandwidth) {
+            if (endptr != hbw_nodes_env && node < num_bandwidth) {
                 bandwidth.push_back(2);
             }
         }
